/**
 * editor core.
 * Wrap所有编辑器修改命令。
 */

//
// Init  
//
var customCommandStack = null;
function InitCustomCommandStack() {
	customCommandStack = new Undo.Stack();
}

//
// Execute custom command.
//
function ExecuteCustomCommand(command) {
	var ret = customCommandStack.execute(command);
	if (ret) {
		document.execCommand('RegisterCustomCommand');	
	}
}

///////// Wrapper /////////

//
// Undo.
//
function ExecuteUndo() {
	if (document.execCommand('UndoCustomCommand')) {
		// handle by myself.
		if (customCommandStack.canUndo()) {
			customCommandStack.undo();
		}
	}
	document.execCommand('undo');

	// 由于undo的时候可能会插入新的表格，所以需要载check一次Table
	TableInit.initAllTables();
}

//
// Redo.
//
function ExecuteRedo() {
	if (document.execCommand('RedoCustomCommand')) {
		// handle by myself.
		if (customCommandStack.canRedo()) {
			customCommandStack.redo();
		}
	}
	document.execCommand('redo');

	// 由于redo的时候可能会插入新的表格，所以需要载check一次Table
	TableInit.initAllTables();
}

// Copy.
function ExecuteCopy() {
	document.execCommand('Copy');
}
// Cut.
function ExecuteCut() {
	document.execCommand('Cut');
}
// Delete.
function ExecuteDelete() {
	document.execCommand('Delete');
}
// Select all.
function ExecuteSelectAll() {
	document.execCommand('SelectAll');
}
// Paste normal.
function ExecutePaste() {
	document.execCommand('Paste');
}
// Bold.
function ExecuteBold() {
	document.execCommand('Bold');
}
// Italic.
function ExecuteItalic() {
	document.execCommand('Italic');
}
// UnderLine.
function ExecuteUnderLine() {
	document.execCommand('UnderLine');
}
// Strikethrough.
function ExecuteStrikeThrough() {
	document.execCommand('StrikeThrough');
}
// JustifyLeft.
function ExecuteJustifyLeft() {
	document.execCommand('JustifyLeft');
}
// JustifyCenter.
function ExecuteJustifyCenter() {
	document.execCommand('JustifyCenter');
}
// JustifyRight.
function ExecuteJustifyRight() {
	document.execCommand('JustifyRight');
}
// ForeColor.
function ExecuteForeColor(color) {
	document.execCommand('ForeColor', false, color);
}
// BackColor.
function ExecuteBackColor(color) {
	document.execCommand('BackColor', false, color);
}
// Indent.
function ExecuteIndent() {
	document.execCommand('Indent');
}
// Outdent.
function ExecuteOutdent() {
	document.execCommand('Outdent');
}
// InsertOrderedList.
function ExecuteInsertOrderedList() {
	document.execCommand('InsertOrderedList');
}
// InsertUnorderedList.
function ExecuteInsertUnorderedList() {
	document.execCommand('InsertUnorderedList');
}
// FontName.
function ExecuteFontName(fontName) {
	document.execCommand('FontName', false, fontName);
}
// FoxFontSize.
function ExecuteFoxFontSize(fontSize) {
	document.execCommand('FoxFontSize', false, fontSize);
}
// CreateLink.
function ExecuteCreateLink(href) {
	//
	// 如果无选中，插入link后将光标置于后面。
	//
	var selection = window.getSelection();
	var isCollapsed = selection.isCollapsed;
	document.execCommand('CreateLink', false, href);
	if (selection && isCollapsed) {
		selection.collapseToEnd();
	}
}
// UnLink.
function ExecuteUnLink() {
	document.execCommand('UnLink');
}
// InsertImage.
function ExecuteInsertImage(src) {
	document.execCommand('InsertImage', false, src);
}
// FormatBlock.
function ExecuteFormatBlock(tagName) {
	document.execCommand('FormatBlock', false, tagName);
}
// SuperScript.
function ExecuteSuperScript() {
	document.execCommand('SuperScript');
}
// SubScript.
function ExecuteSubScript() {
	document.execCommand('SubScript');
}
// RemoveFormat.
function ExecuteRemoveFormat() {
	document.execCommand('RemoveFormat');
}
// InsertHorizontalRule.
function ExecuteInsertHorizontalRule() {
	document.execCommand('InsertHorizontalRule');
}
// ApplyCssStyle.
function ExecuteApplyCssStyle(styles) {
	document.execCommand('ApplyCssStyle', false, styles);
}
// Paste HTML.
function ExecutePasteHTML(html, bTransform, selected, scrollToView, strWithFocus) {
	function SetFocusToNode(node) {
		if (!node) return;
		var range = document.createRange();
		range.setStart(node);
		sel = window.getSelection();
		sel.removeAllRanges();
		sel.addRange(range);
	}
	try {
		html = ApplyInlineStyle(html, null, bTransform).html;
		document.execCommand("InsertHtml", false, html);
		if (strWithFocus !== '') {
			var el = $('#' + strWithFocus);
			var p = el.parent();
			p.html(el.html());
			SetFocusToNode(p[0], 2);
		}
	} catch (e) {}
}
// Paste text.
function ExecutePasteText(text, selected, scrollToView) {
	var nodeText = FoxUtils.TextToHtml(text);
	var fontStyle = GetCurrentFullFontStyle();
	var css = GetCSSStyleTextFromStyleItem(fontStyle);
	var html = '<span style=\'' + css + '\'>' + nodeText + '</span>';

	ExecutePasteHTML(html, true, selected, scrollToView, '');
}
// Paste tab.
function ExecutePasteTab() {
	var fontStyle = GetCurrentFullFontStyle();
	var css = GetCSSStyleTextFromStyleItem(fontStyle);
	var html = '<span style=\'white-space: pre;' + css + '\'>    </span>';

	new ExecutePasteHTML(html, true, false, false, '');
}
// Resize image.
function ExecuteResizeImage(image, width, height) {
	ExecuteCustomCommand(new ResizeImageCommand(image, width, height));
}
// Insert table.
function ExecuteInsertTable() {
	function getTdHtml(options) {
		var tdStyle = options.wordBreak ? "word-break: break-all;" : "";
		tdStyle = tdStyle + " width: " + (options.cellWidth - 2 * options.cellPadding) + ";";
		tdStyle = tdStyle + " padding: " + options.cellPadding + "px;";
		tdStyle = tdStyle + " border: " + options.borderWidth + " " + options.borderType + " " + options.borderColor + ";";
		tdStyle = tdStyle + " background-color: " + options.backgroundColor + ";";
		return "<td style='" + tdStyle + "'>&#8203;</td>";
	}

	function getTableHtml(options) {
		var caption = "<caption>" + options.caption + "</caption>";
		
		var tableStyle = "outline: none; ";
		if (options.cellSpacing) 
			tableStyle = tableStyle + " border-spacing: " + options.cellSpacing + ";";
		else
			tableStyle = tableStyle + " border-collapse: collapse;";

		var tableWidth = parseInt(options.tableWidth);
		if (/\d+%$/i.test(options.tableWidth)) {
			tableWidth = $('body').innerWidth() * tableWidth / 100;
		}
		options.cellWidth = tableWidth / Number(options.column);

		var tr = '<tr>';
		for (var i = 0; i < options.column; i++) {
			tr = tr + getTdHtml(options);
		}

		var trs = '';
		for (var i = 0; i < options.row; i++) {
			trs = trs + tr;
		}

		// todo. cellPadding需要更好去处理
		return "<table class='lastInsertTable' style='" + tableStyle + "' contenteditable='true'>" + caption + trs + "</table>";
	}

	// 为了支持表格操作，需要wrap一些标签。
	var options = {
		caption: arguments[0],
		tableWidth: arguments[1],
		row: arguments[2],
		column: arguments[3],
		cellSpacing: arguments[4],
		cellPadding: arguments[5] + 2,
		backgroundColor: arguments[6],
		borderWidth: arguments[7],
		borderType: arguments[8],
		borderColor: arguments[9],
		wordBreak: arguments[10]
	};

	var tableHtml = getTableHtml(options);
	
	document.execCommand("InsertHtml", false, tableHtml);

	var lastInsertTable = $('.lastInsertTable').removeClass('lastInsertTable');
	TableInit.initTable(lastInsertTable);

	setTimeout(function() {
		var firstCell = lastInsertTable.find('td').first();
		if (firstCell.length !== 0) {
			window.getSelection().collapse(firstCell[0], 0);
		}
	}, 10);
}
// Delete table.
function ExecuteDeleteTable() {
	ExecuteCustomCommand(new DeleteTableCommand());
}
// Insert table caption.
function ExecuteInsertTableCaption() {
	ExecuteCustomCommand(new InsertTableCaptionCommand());
}
// Delete table caption.
function ExecuteDeleteTableCaption() {
	ExecuteCustomCommand(new DeleteTableCaptionCommand());
}
// Insert header row.
function ExecuteInsertHeaderRow() {
	ExecuteCustomCommand(new InsertHeaderRowCommand());
}
// Delete header row.
function ExecuteDeleteHeaderRow() {
	ExecuteCustomCommand(new DeleteHeaderRowCommand());
}
// Insert header column.
function ExecuteInsertHeaderColumn() {
	ExecuteCustomCommand(new InsertHeaderColumnCommand());
}
// Delete Current Row.
function ExecuteDeleteCurrentRow() {
	ExecuteCustomCommand(new DeleteCurrentRowCommand());
}
// Delete Current Column.
function ExecuteDeleteCurrentColumn() {
	ExecuteCustomCommand(new DeleteCurrentColumnCommand());
}
// Delete header column.
function ExecuteDeleteHeaderColumn() {
	ExecuteCustomCommand(new DeleteHeaderColumnCommand());
}
// Insert Top Row.
function ExecuteInsertTopRow() {
	ExecuteCustomCommand(new InsertTopRowCommand());
}
// Insert Bottom Row.
function ExecuteInsertBottomRow() {
	ExecuteCustomCommand(new InsertBottomRowCommand());
}
// Insert Left Column.
function ExecuteInsertLeftColumn() {
	ExecuteCustomCommand(new InsertLeftColumnCommand());
}
// Insert Right Column.
function ExecuteInsertRightColumn() {
	ExecuteCustomCommand(new InsertRightColumnCommand());
}
// Merge Cell.
function ExecuteMergeCell() {
	ExecuteCustomCommand(new MergeCellCommand());
}
// Split Cell.
function ExecuteSplitCell() {
	ExecuteCustomCommand(new SplitCellCommand());
}
// Interlace Color for Rows.
function ExecuteInterlaceRowColor() {
	ExecuteCustomCommand(new InterlaceRowColorCommand());
}