/**
 * table edit function.
 * base on jquery.
 */

var TableInit = {
	containerBeginHtml: "<div class='table-container' style='display: inline-block;' contenteditable='false'>",
	containerEndHtml: "</div>",
	rowResizerHtml: "<div><div class='table-row-resize-dragger'></div><div class='table-row-resize-guide' style='display: none;''></div></div>",
	columnResizerHtml: "<div><div class='table-column-resize-dragger'></div><div class='table-column-resize-guide' style='display: none;''></div></div>",
	selectorHtml: "<div class='table-row-selector'></div><div class='table-column-selector'></div>",
	tailHtml: "<span>&#8203;</span>",

	//
	// 在table外层包装table-container
	//
	wrapTableContainer: function(table) {
		table.wrap(this.containerBeginHtml + this.containerEndHtml);
		table.after(this.rowResizerHtml + this.columnResizerHtml + this.selectorHtml);
	},

	checkContenteditable: function(table) {
		var tableContainer = table.closest('.table-container');
		
		if (tableContainer.attr('contenteditable') !== 'false') {
			tableContainer.attr('contenteditable', 'false');
			tableContainer.find('table').attr('contenteditable', 'true');
		}
	},

	//
	// 初始化所有的表格，加上必要的数据
	//
	initAllTables: function() {
		var that = this;
		$('table').each(function() {
			that.initTable($(this));
		});
	},

	//
	// 初始化指定表格，加上必要的数据
	//
	initTable: function(table) {
		if (table.length == 0) return;

		table.find('tr').children().each(function() {
			var cell = $(this);
			if (cell.attr('rowspan') == undefined) cell.attr('rowspan', 1);
			if (cell.attr('colspan') == undefined) cell.attr('colspan', 1);
		});

		var tableContainer = table.closest('.table-container');

		if (tableContainer.length == 0) {
			this.wrapTableContainer(table);
			table.attr('contenteditable', 'true');
			table.css('outline', 'none');
			tableContainer = table.closest('.table-container');
		}

		if (!tableContainer.data('TableCore')) {
			var tableCore = new TableCore(tableContainer);
			tableCore.setTableMatrix();
		}

		this.checkContenteditable(table);
	}
};

//
// Table core.
// 维护表格的单元格矩阵。
//
(function() {
	var TableCore = this.TableCore = function(tableContainer) {
		this.tableContainer = tableContainer;	// table container.
		this.matrix = [];
	};
	TableCore.prototype = {
		// 
		// 返回JQuery包装的空元素
		// 
		NoneElement: function() {
			return $('div').filter('td');
		},

		//
		// 获取x, y所在的cell。
		// 返回包含零个或一个元素的 jQuery 对象。
		//
		getCellByPoint: function(x, y) {
			var target = $(document.elementFromPoint(x, y));
			var cell = target.closest('td');
			if (cell.length == 0)
				cell = target.closest('th');
			return cell;
		},

		//
		// 返回表格的单元格矩阵。
		// 并在td元素的data中设置单元格的坐标信息。
		//
		setTableMatrix: function() {
			this.matrix = [];

			var trs = this.tableContainer.find('tr');

			// 计算有多少单元格列
			var columnLength = 0;
			trs.first().children().each(function() {	// th + td
				var colspan = Number($(this).attr('colspan'));
				columnLength += colspan;
			});

			var that = this;
			trs.each(function() {
				that.matrix[that.matrix.length] = new Array(columnLength);
			});

			// 计算单元格排列矩阵
			// 这里还是蛮复杂的
			for (var row = 0; row < trs.length; row++) {
				var cells = $(trs[row]).children();	// th + td
				var column = 0;
				for (var i = 0; i < cells.length; i++) {
					while (column < columnLength) {
						if (this.matrix[row][column] == undefined) break;
						column++;
					}
					if (column == columnLength) break;

					var cell = $(cells[i]);

					var rowspan = Number(cell.attr('rowspan'));
					var colspan = Number(cell.attr('colspan'));

					for (var m = 0; m < rowspan; m++) {
						for (var n = 0; n < colspan; n++) {
							this.matrix[row + m][column + n] = cell;
						}
					}

					cell.data('coord', {
						row: row,
						column: column,
						rowspan: rowspan,
						colspan: colspan
					});
				}
			}
			this.tableContainer.data('TableCore', this);
		},

		//
		// 清除
		//
		clearTableMatrix: function() {
			this.tableContainer.data('TableCore', '');
			this.tableContainer = null;
			this.matrix = [];
		},

		// //
		// // 获取cell在第几行，跨几行
		// // base 0.
		// //
		// getCoordInRow: function(cell) {
		// 	var coord = cell.data('coord');
		// 	return {
		// 		from: coord.column,
		// 		colspan: coord.colspan
		// 	};
		// },

		// //
		// // 获取td在第几列，跨几列
		// // base 0.
		// //
		// getCoordInColumn: function(cell) {
		// 	var coord = cell.data('coord');
		// 	return {
		// 		from: coord.row,
		// 		rowspan: coord.rowspan
		// 	};
		// },

		//
		// 获取cell在表格中处于第几行，跨几行，第几列，跨几列。
		// base 0.
		//
		getCoordInTable: function(cell) {
			var coord = cell.data('coord');
			return coord;
		},

		// state:
		// 	handling 为元素正在待探测队列中
		// 	handled 为元素已经属于surround中
		stateHandling: 'handling',
		stateHandled: 'handled',

		//
		// 返回包含cell1, cell2的最小矩阵中的(th or td)对象集。
		//
		getSurroundCells: function(cell1, cell2) {
			if (cell1.length == 0 || cell2.length == 0) return;

			var that = this;

			var coord1 = cell1.data('coord');
			var coord2 = cell2.data('coord');

			// surround为最小矩形。
			var surround = {
				minRow: 9999,
				maxRow: 0,
				minColumn: 9999,
				maxColumn: 0
			};
			
			// 更新surround区域
			function updateSurround(coord) {
				if (coord.row < surround.minRow) surround.minRow = coord.row;
				if (coord.column < surround.minColumn) surround.minColumn = coord.column;
				if (coord.row + coord.rowspan - 1 > surround.maxRow) surround.maxRow = coord.row + coord.rowspan - 1;
				if (coord.column + coord.colspan - 1 > surround.maxColumn) surround.maxColumn = coord.column + coord.colspan - 1;
			}

			updateSurround(coord1);
			updateSurround(coord2);

			// 待处理的cells
			var handlingList = [];

			// 查看元素是否处于当前surround中
			function insideSurround(coord) {
				function inside(row, column) {
					if (row < surround.minRow) return false;
					if (row > surround.maxRow) return false;
					if (column < surround.minColumn) return false;
					if (column > surround.maxColumn) return false;
					return true;
				}
				if (inside(coord.row, coord.column)) return true;
				if (inside(coord.row, coord.column + coord.colspan - 1)) return true;
				if (inside(coord.row + coord.rowspan - 1, coord.column)) return true;
				if (inside(coord.row, coord.column + coord.colspan - 1)) return true;
				return false;
			}

			function PutCellToHandling(cell) {
				if (cell == undefined) return;
				if (cell.data('state') == that.stateHandled) return;
				if (cell.data('state') == that.stateHandling) return;

				var coord = cell.data('coord');

				if (insideSurround(coord)) {
					cell.data('state', 'handling');
					handlingList.push(cell);

					updateSurround(coord); // 更新surround
				}
			}

			PutCellToHandling(cell1);
			PutCellToHandling(cell2);

			// 查找指定行，处于surround中，但是需要继续探测的元素
			function findShouldHandleCellFromRow(row, column, colspan) {
				if (row < surround.minRow || row > surround.maxRow) return;

				for (var j = column; j < column + colspan; j++) {
					PutCellToHandling(that.matrix[row][j]);
				}
			}

			// 查找指定列，处于surround中，但是需要继续探测的元素
			function findShouldHandleCellFromColumn(column, row, rowspan) {
				if (column < surround.minColumn || column > surround.maxColumn) return;

				for (var i = row; i < row + rowspan; i++) {
					PutCellToHandling(that.matrix[i][column]);
				}
			}

			var cells = this.NoneElement();

			// 循环探测处于handling队列中的元素。
			while (handlingList.length > 0) {
				var handlingCell = handlingList.pop();
				var coord = handlingCell.data('coord');

				// 找出周边需要处理的td
				findShouldHandleCellFromRow(coord.row - 1, coord.column, coord.colspan); // north
				findShouldHandleCellFromRow(coord.row + 1, coord.column, coord.colspan); // south
				findShouldHandleCellFromColumn(coord.column - 1, coord.row, coord.rowspan); // west
				findShouldHandleCellFromColumn(coord.column + 1, coord.row, coord.rowspan); // east

				cells = $.merge(cells, handlingCell);
				handlingCell.data('state', 'handled');
			}

			// 探测完毕，置空state。
			cells.data('state', '');
			return cells;
		},

		//
		// 通过cell获取同一横排的cell
		//
		getFatRowByCell: function(cell) {
			var coord = cell.data('coord');
			var column = 0;
			var begin = this.matrix[coord.row][column];
			column = this.matrix[coord.row].length - 1;
			var end = this.matrix[coord.row][column];
			var cells = this.getSurroundCells(begin, end);
			return cells;
		},

		//
		// 通过x, y获取整个横排的cell
		//
		getFatRowByPoint: function(x, y) {
			var cell = this.getCellByPoint(x, y);
			return this.getFatRowByCell(cell);
		},

		//
		// 通过td获取同一纵排的cell
		//
		getFatColumnByCell: function(cell) {
			var coord = cell.data('coord');
			var row = 0;
			var begin = this.matrix[row][coord.column];
			row = this.matrix.length - 1;
			var end = this.matrix[row][coord.column];
			var cells = this.getSurroundCells(begin, end);
			return cells;
		},

		//
		// 通过td获取同一纵排的td。
		//
		getFatColumnByPoint: function(x, y) {
			var cell = this.getCellByPoint(x, y);
			return this.getColumnByCell(cell);
		},

		//
		// ??
		//
		getTwinRowBetween: function(topCell, bottomCell) {
			var topRow = this.NoneElement();
			var bottomRow = this.NoneElement();
			var coord1 = topCell.data('coord');
			var coord2 = bottomCell.data('coord');
			var row1 = coord1.row + coord1.rowspan - 1;
			var row2 = coord2.row;

			// left
			for (var column = coord1.column; column >= 0; column--) {
				var cell = this.matrix[row1][column];
				var coord = cell.data('coord');
				if (coord.row + coord.rowspan == row2) {
					topRow = $.merge(cell, topRow);
					bottomRow = $.merge(this.matrix[row2][column], bottomRow);
				} else {
					break;
				}
			}
			// right
			for (var column = coord1.column + 1; column < this.matrix[row1].length; column++) {
				var cell = this.matrix[row1][column];
				var coord = cell.data('coord');
				if (coord.row + coord.rowspan == row2) {
					topRow = $.merge(topRow, cell);
					bottomRow = $.merge(bottomRow, this.matrix[row2][column]);
				} else {
					break;
				}
			}
			return {
				top: topRow,
				bottom: bottomRow
			};
		},

		//
		// ??
		//
		getTwinColumnBetween: function(leftCell, rightCell) {
			var leftRow = this.NoneElement();
			var rightRow = this.NoneElement();
			var coord1 = leftCell.data('coord');
			var coord2 = rightCell.data('coord');
			var column1 = coord1.column + coord1.colspan - 1;
			var column2 = coord2.column;

			// top
			for (var row = coord1.row; row >= 0; row--) {
				var cell = this.matrix[row][column1];
				var coord = cell.data('coord');
				if (coord.column + coord.colspan == column2) {
					leftRow = $.merge(cell, leftRow);
					rightRow = $.merge(this.matrix[row][column2], rightRow);
				} else {
					break;
				}
			}
			// bottom
			for (var row = coord1.row + 1; row < this.matrix.length; row++) {
				var cell = this.matrix[row][column1];
				var coord = cell.data('coord');
				if (coord.column + coord.colspan == column2) {
					leftRow = $.merge(leftRow, cell);
					rightRow = $.merge(rightRow, this.matrix[row][column2]);
				} else {
					break;
				}
			}
			return {
				left: leftRow,
				right: rightRow
			};
		},

		getThinRowByCoord: function(coord) {
			var rowCells = this.NoneElement();
			var lastCell = undefined;
			for (var column = 0; column < this.matrix[coord.row].length; column++) {
				var cell = this.matrix[coord.row][column];
				if (lastCell == undefined || lastCell[0] !== cell[0]) {
					rowCells = $.merge(rowCells, cell);
				}
				lastCell = cell;
			}
			return rowCells;
		},

		getThinColumnByCoord: function(coord) {
			var columnTds = this.NoneElement();
			var lastTd = undefined;
			for (var row = 0; row < this.matrix.length; row++) {
				var td = this.matrix[row][coord.column];
				if (lastTd == undefined || lastTd[0] !== td[0]) {
					columnTds = $.merge(columnTds, td);
				}
				lastTd = td;
			}
			return columnTds;
		}
	};
}).call(this);

//
// 选中单元格
//
var TableSelect = {
	selectClass: 'selected-cell',
	
	init: function() {
		$(document).delegate('td', 'mousedown', $.proxy(this.startDetectSelection, this));
		$(document).delegate('th', 'mousedown', $.proxy(this.startDetectSelection, this));
		$(document).bind('mouseup', $.proxy(this.clearWhenMouseUp, this));
		$(document).delegate('.table-row-selector', 'mousedown', $.proxy(this.selectRow, this));
		$(document).delegate('.table-column-selector', 'mousedown', $.proxy(this.selectColumn, this));
	},

	uninit: function() {
		$(document).undelegate('td', 'mousedown', this.startDetectSelection);
		$(document).undelegate('th', 'mousedown', this.startDetectSelection);
		$(document).unbind('mouseup', this.clearWhenMouseUp);
		$(document).undelegate('.table-row-selector', 'mousedown', this.selectRow);
		$(document).undelegate('.table-column-selector', 'mousedown', this.selectColumn);
	},

	isDisableSelection: false,
	disableSelection: function() {
		this.isDisableSelection = true;
		window.getSelection().removeAllRanges();
		$('body').addClass('unselectable');
	},

	enableSelection: function() {
		if (this.isDisableSelection) {
			var selection = window.getSelection();
			if (selection && selection.rangeCount) {
				selection.collapseToStart();
			}
			$('body').removeClass('unselectable');
			this.isDisableSelection = false;	
		}
	},

	startCell: null,
	tableContainer: null,
	startDetectSelection: function(ev) {
		if (ev.which !== 1) return;

		this.startCell = $(ev.target).closest('td');
		if (this.startCell.length == 0) 
			this.startCell = $(ev.target).closest('th');

		this.tableContainer = this.startCell.closest('.table-container');
		this.tableContainer.find('.' + this.selectClass).removeClass(this.selectClass);

		this.startCell.bind('mouseleave', $.proxy(this.disableSelection, this));
		if (this.startCell.is('td'))
			this.tableContainer.delegate('td', 'mouseenter', $.proxy(this.detectingSelection, this));
		else
			this.tableContainer.delegate('th', 'mouseenter', $.proxy(this.detectingSelection, this));	
		$(document).bind('mouseup', $.proxy(this.endDetectSelection, this));
	},

	detectingSelection: function(ev) {
		var target = $(ev.target).closest('td');
		if (target.length == 0)
			target = $(ev.target).closest('th');
		if (target.length == 0) return;
		window.getSelection().collapse(target[0], 0);

		var tableCore = this.tableContainer.data('TableCore');
		var selection = tableCore.getSurroundCells(this.startCell, target);
		if (selection.length !== 0) {
			this.tableContainer.find('.' + this.selectClass).removeClass(this.selectClass);
			selection.addClass(this.selectClass);
		}
	},

	endDetectSelection: function(ev) {
		if (ev.which !== 1) return;

		this.tableContainer.undelegate('td', 'mouseenter', this.detectingSelection);
		this.tableContainer.undelegate('th', 'mouseenter', this.detectingSelection);
		$(document).unbind('mouseup', this.endDetectSelection);
		$('body').unbind("selectstart", this.disableSelection);
		
		this.startCell.unbind('mouseleave', this.disableSelection);
		this.enableSelection();
	},

	selectRow: function(ev) {
		var tableContainer = $(ev.target).closest('.table-container');
		tableContainer.find('.' + this.selectClass).removeClass(this.selectClass);
		var tableCore = tableContainer.data('TableCore');
		var cell = tableCore.getCellByPoint(ev.pageX + 5, ev.pageY);
		tableCore.getFatRowByCell(cell).addClass(this.selectClass);
	},

	selectColumn: function(ev) {
		var tableContainer = $(ev.target).closest('.table-container');
		tableContainer.find('.' + this.selectClass).removeClass(this.selectClass);
		var tableCore = tableContainer.data('TableCore');
		var cell = tableCore.getCellByPoint(ev.pageX, ev.pageY + 5);
		tableCore.getFatColumnByCell(cell).addClass(this.selectClass);
	}
};